             .title        "Irem H3001 (INES 65) Plugin"

;10/01/09
;BootGod

             ;vectors for standard system calls

send_byte:   .equ 0200h
baton:       .equ 0203h
chk_vram:    .equ 0206h
chk_wram:    .equ 0209h
wr_ppu:      .equ 020ch
read_byte:   .equ 020fh
init_crc:    .equ 0212h
do_crc:      .equ 0215h
finish_crc:  .equ 0218h

crc0:        .equ 0080h
crc1:        .equ 0081h
crc2:        .equ 0082h
crc3:        .equ 0083h

address:       .equ 00e0h
address_lo:    .equ 00e0h
address_hi:    .equ 00e1h
curbank:    .equ 00e2h
banksleft:    .equ 00e3h
bank0crc:  .equ 00e4h

tx:       .equ 00e8h
reg_lo:    .equ 00e8h
reg_hi:    .equ 00e9h

numpages_lo .equ 00eah
numpages_hi .equ 00ebh


temp_crc:    .equ 00ech

;plugin header defines
p_ver: = 1	;plugin version
p_read: = 1	;plugin is for downloading (dumping)
p_write: = 2	;plugin is for uploading (RAM carts / WRAM etc)
p_other: = 3	;plugin is for some other purpose (i.e. BankWatch)
p_var1: = 4	;plugin uses user variable #1
p_var2: = 8	;plugin uses user variable #2
p_var3: = 16	;plugin uses user variable #3
p_var4: = 32	;plugin uses user variable #4

;plugin header data

             .org 0380h
             
             ;plugin description
             .db "Irem H3001 Carts"
             .fill 03e0h-*,00h

             ;plugin creator(s)
             .db "BootGod"
             .fill 03f8h-*,00h

             ;PRG/CHR/WRAM min/max sizes supported
             .db 128 >> 4,256 >> 4
             .db 128 >> 3,256 >> 3
             .db 0,0

             ;plugin version and flags
             .db p_ver
             .db p_read

;plugin code start here

             
;mirroring is mapper controlled

             lda #4
             jsr send_byte

;get crc of PRG bank 0 (8K)

             lda #0
             jsr wr_pbank	;set to PRG bank 0
             jsr crc_pbank

             ldx #3
ld_pcrc:     lda temp_crc,x	;save for comparision
             sta bank0crc,x
             dex
             bpl ld_pcrc

;start out crc check at bank 16	;128K in
             lda #16
             sta curbank
             lda #2
             sta numpages_hi
             lda #0
             sta numpages_lo

next_pcrc:   lda curbank
             jsr wr_pbank	;set PRG bank num
             jsr crc_pbank
             
;compare crc's of bank 0 and curbank
             ldx #3
chk_crc:     lda bank0crc,x
             cmp temp_crc,x
             bne crc_fail
             dex
             bpl chk_crc

             jmp got_prg

crc_fail:    lda curbank	;check to see if we've hit the max bank num
             cmp #32
             beq got_prg

             asl numpages_hi	;size in pages to send to pc
             asl curbank
             jmp next_pcrc

;send size info

got_prg:     lda curbank	;setup bank counters for dumping
             sta banksleft
             lda #0
             sta curbank

             lda numpages_lo	;send PRG header
             jsr send_byte
             lda numpages_hi
             jsr send_byte
             lda #001h
             jsr send_byte

next_prg:    lda curbank
             jsr wr_pbank	;write cur bank num
             inc curbank

             lda #0
             sta address_lo
             lda #080h
             sta address_hi	;set address to $8000

             ldy #0
             ldx #32		;setup for 8K bank dump

dump_prg:    lda (address),y
             jsr send_byte
             iny
             bne dump_prg
             inc address_hi
             dex
             bne dump_prg
             dec banksleft
             bne next_prg
             


;determine CHR ROM size

             lda #0
             jsr wr_cbank	;set to CHR bank 0
             jsr crc_cbank

             ldx #3
ld_ccrc:     lda temp_crc,x	;save for comparision
             sta bank0crc,x
             dex
             bpl ld_ccrc
             
;start out crc check at bank 128 (128K in)
             lda #128
             sta curbank
             lda #2
             sta numpages_hi
             lda #0
             sta numpages_lo

next_ccrc:   lda curbank
             jsr wr_cbank	;set CHR bank num
             jsr crc_cbank
             
;compare crc's of bank 0 and curbank
             ldx #3
chk_ccrc:    lda bank0crc,x
             cmp temp_crc,x
             bne crc_cfail
             dex
             bpl chk_ccrc

             jmp got_chr

crc_cfail:   lda curbank	;check to see if we've hit the max bank num (by wrapping back to 0)
             cmp #0
             beq got_chr

             asl numpages_hi	;size in pages to send to pc
             asl curbank
             jmp next_ccrc
             
got_chr:     lda curbank	;setup bank counters for dumping
             sta banksleft
             lda #0
             sta curbank

             lda numpages_lo	;send CHR header
             jsr send_byte
             lda numpages_hi
             jsr send_byte
             lda #002h
             jsr send_byte
             
next_chr:    lda curbank
             jsr wr_cbank
             inc curbank
             lda #0
             jsr wr_ppu
             lda 2007h

             ldx #4		;setup for 1K bank dump
             ldy #0

dump_chr:    lda 2007h
             jsr send_byte
             iny
             bne dump_chr
             dex
             bne dump_chr
             dec banksleft
             bne next_chr

finished:    lda #000h		;send termination header
             jsr send_byte
             lda #000h
             jsr send_byte
             lda #000h
             jsr send_byte

             rts

wr_pbank:    sta 8000h
             rts

wr_cbank:    sta 0b000h
             rts

;CRC routine for 8K PRG bank @ $8000, stores crc in temp_crc

crc_pbank:   lda #0
             sta address_lo
             lda #080h
             sta address_hi	;set address to $8000

             jsr init_crc
             ldy #0
             ldx #32		;init crc for 8K of data (32 pages = 8K)

crc_ploop:   lda (address),y	;crc generation loop
             stx tx
             jsr do_crc
             ldx tx
             iny
             bne crc_ploop
             inc address_hi
             dex
             bne crc_ploop

             jsr finish_crc	;crc done

             ldx #3
crc_psave:   lda crc0,x		;save crc in temp_crc
             sta temp_crc,x
             dex
             bpl crc_psave

             rts

;CRC routine for 1K CHR bank @ $0000, stores crc in temp_crc

crc_cbank:   lda #0
             jsr wr_ppu
             lda 2007h		;set PPU address to $0000

             jsr init_crc
             ldx #4
             ldy #0		;init crc for 1K of data (4 pages = 1K)

crc_cloop:   lda 2007h		;crc generation loop
             stx tx
             jsr do_crc
             ldx tx
             iny
             bne crc_cloop
             dex
             bne crc_cloop
             
             jsr finish_crc	;crc done

             ldx #3
crc_csave:   lda crc0,x		;save crc in temp_crc
             sta temp_crc,x
             dex
             bpl crc_csave

             rts



             .fill 0800h-*,0ffh   ;fill rest to get 1K of data

             .end
